package com.sun.xml.internal.ws.wsdl;

import com.sun.xml.internal.ws.api.message.Message;
import com.sun.xml.internal.ws.api.message.MessageHeaders;
import com.sun.xml.internal.ws.api.message.Messages;
import com.sun.xml.internal.ws.api.message.AddressingUtils;
import com.sun.xml.internal.ws.api.message.Packet;
import javax.xml.namespace.QName;
import java.util.Iterator;
import com.sun.xml.internal.ws.model.JavaMethodImpl;
import com.sun.xml.internal.ws.model.AbstractSEIModelImpl;
import com.sun.xml.internal.ws.resources.AddressingMessages;
import com.sun.xml.internal.ws.api.model.wsdl.WSDLBoundOperation;
import java.util.HashMap;
import com.sun.istack.internal.Nullable;
import com.sun.xml.internal.ws.api.model.SEIModel;
import com.sun.xml.internal.ws.api.WSBinding;
import com.sun.xml.internal.ws.api.model.wsdl.WSDLPort;
import com.sun.istack.internal.NotNull;
import com.sun.xml.internal.ws.api.addressing.AddressingVersion;
import com.sun.xml.internal.ws.api.model.WSDLOperationMapping;
import java.util.Map;
import java.util.logging.Logger;

final class ActionBasedOperationFinder extends WSDLOperationFinder
{
    private static final Logger LOGGER;
    private final Map<ActionBasedOperationSignature, WSDLOperationMapping> uniqueOpSignatureMap;
    private final Map<String, WSDLOperationMapping> actionMap;
    @NotNull
    private final AddressingVersion av;
    
    public ActionBasedOperationFinder(WSDLPort wsdlModel, WSBinding binding, @Nullable SEIModel seiModel) {
        super(wsdlModel, binding, seiModel);
        assert binding.getAddressingVersion() != null;
        this.av = binding.getAddressingVersion();
        this.uniqueOpSignatureMap = new HashMap();
        this.actionMap = new HashMap();
        if (seiModel != null) {
            for (JavaMethodImpl m : ((AbstractSEIModelImpl)seiModel).getJavaMethods()) {
                if (m.getMEP().isAsync) {
                    continue;
                }
                String action2 = m.getInputAction();
                QName payloadName2 = m.getRequestPayloadName();
                if (payloadName2 == null) {
                    payloadName2 = PayloadQNameBasedOperationFinder.EMPTY_PAYLOAD;
                }
                if ((action2 == null || action2.equals("")) && m.getOperation() != null) {
                    action2 = m.getOperation().getOperation().getInput().getAction();
                }
                if (action2 != null) {
                    ActionBasedOperationSignature opSignature = new ActionBasedOperationSignature(action2, payloadName2);
                    if (this.uniqueOpSignatureMap.get(opSignature) != null) {
                        ActionBasedOperationFinder.LOGGER.warning(AddressingMessages.NON_UNIQUE_OPERATION_SIGNATURE(this.uniqueOpSignatureMap.get(opSignature), m.getOperationQName(), action2, payloadName2));
                    }
                    this.uniqueOpSignatureMap.put(opSignature, this.wsdlOperationMapping(m));
                    this.actionMap.put(action2, this.wsdlOperationMapping(m));
                }
                else {
                    continue;
                }
            }
        }
        else {
            for (WSDLBoundOperation wsdlOp : wsdlModel.getBinding().getBindingOperations()) {
                QName payloadName = wsdlOp.getRequestPayloadName();
                if (payloadName == null) {
                    payloadName = PayloadQNameBasedOperationFinder.EMPTY_PAYLOAD;
                }
                String action = wsdlOp.getOperation().getInput().getAction();
                ActionBasedOperationSignature opSignature = new ActionBasedOperationSignature(action, payloadName);
                if (this.uniqueOpSignatureMap.get(opSignature) != null) {
                    ActionBasedOperationFinder.LOGGER.warning(AddressingMessages.NON_UNIQUE_OPERATION_SIGNATURE(this.uniqueOpSignatureMap.get(opSignature), wsdlOp.getName(), action, payloadName));
                }
                this.uniqueOpSignatureMap.put(opSignature, this.wsdlOperationMapping(wsdlOp));
                this.actionMap.put(action, this.wsdlOperationMapping(wsdlOp));
            }
        }
        return;
    }
    
    @Override
    public WSDLOperationMapping getWSDLOperationMapping(Packet request) throws DispatchException {
        MessageHeaders hl = request.getMessage().getHeaders();
        String action = AddressingUtils.getAction(hl, this.av, this.binding.getSOAPVersion());
        if (action == null) {
            return null;
        }
        Message message = request.getMessage();
        String localPart = message.getPayloadLocalPart();
        QName payloadName;
        if (localPart == null) {
            payloadName = PayloadQNameBasedOperationFinder.EMPTY_PAYLOAD;
        }
        else {
            String nsUri = message.getPayloadNamespaceURI();
            if (nsUri == null) {
                nsUri = "";
            }
            payloadName = new QName(nsUri, localPart);
        }
        WSDLOperationMapping opMapping = (WSDLOperationMapping)this.uniqueOpSignatureMap.get(new ActionBasedOperationSignature(action, payloadName));
        if (opMapping != null) {
            return opMapping;
        }
        opMapping = (WSDLOperationMapping)this.actionMap.get(action);
        if (opMapping != null) {
            return opMapping;
        }
        Message result = Messages.create(action, this.av, this.binding.getSOAPVersion());
        throw new DispatchException(result);
    }
    
    static {
        LOGGER = Logger.getLogger(ActionBasedOperationFinder.class.getName());
    }
}
